000001 IDENTIFICATION DIVISION.
000002 PROGRAM-ID.  ROSES.
000003
000004*    This is a program to demonstrate a simple database
000005*    application with COBOL12. It is not offered as a
000006*    good programming example, but rather to display some
000007*    of the various contructs available.
000008
000009 ENVIRONMENT DIVISION.
000010 CONFIGURATION SECTION.
000011 SPECIAL-NAMES.
000012     DATE IS "S-".
000013*    A- American, E- European.
000014 INPUT-OUTPUT SECTION.
000015 FILE-CONTROL.
000016*------------
000017     SELECT OPTIONAL ROSE-FILE ASSIGN TO DISK
000018       ORGANIZATION INDEXED ACCESS DYNAMIC
000019       RECORD KEY IS ROSE-NAME
000020       ALTERNATE RECORD KEY IS ROSE-TYPE
000021                 WITH DUPLICATES
000022       ALTERNATE RECORD KEY IS ROSE-COLOUR
000023                 WITH DUPLICATES
000024       FILE STATUS IS ROSE-STATUS.
000025*      Note that the alternate keys will not be in name
000026*      sequence but in the order in which they were added.
000027
000028 DATA DIVISION.
000029 FILE SECTION.
000030 FD  ROSE-FILE
000031*-------------
000032     LABEL RECORDS STANDARD
000033     VALUE OF ID IS "ROSES.DAT".
000034 01  ROSE-RECORD.
000035     05  ROSE-NAME            PIC X(22).
000036     05  ROSE-TYPE            PIC X(14).
000037     05  ROSE-COLOUR          PIC X(6).
000038     05  ROSE-METERS          PIC 99V9.
000039     05  ROSE-FEET            PIC 99V9.
000040     05  ROSE-SEASON          PIC X(6).
000041     05  ROSE-DESCR           PIC X(235).
000042
000043 WORKING-STORAGE SECTION.
000044*-----------------------
000045 01  CURRENT-ROSE-RECORD.
000046*    This is a copy of the current Rose-Record, which is
000047*    used for screen display and updating.
000048     05  CURR-NAME            PIC X(22).
000049     05  CURR-TYPE            PIC X(14).
000050     05  CURR-COLOUR          PIC X(6).
000051     05  CURR-HEIGHT.
000052       10  CURR-METERS        PIC 99V9.
000053       10  CURR-FEET          PIC 99V9.
000054     05  CURR-SEASON          PIC X(6).
000055     05  CURR-DESCR           PIC X(235).
000056
000057 77  ROSE-STATUS              PIC XX. * File status
000058 77  MESSAGE-REPLY            PIC 9999 COMP.
000059 77  FIND-KEY                 PIC X. * Name Type or Colour
000060 77  SAVED-HEIGHT             PIC X(6).
000061 77  TTBLZ                    PIC 9999 COMP.
000062 77  CTBLZ                    PIC 9999 COMP.
000063 77  STBLZ                    PIC 9999 COMP.
000064
000065 77                           PIC X VALUE "N".
000066     88  NO-CURRENT-RECORD          VALUE "N".
000067     88  CURRENT-RECORD             VALUE "C".
000068     88  DELETED-RECORD             VALUE "D".
000069
000070 01  TYPE-TABLE.
000071     05  TYPE-TABLE-DATA.
000072       10  PIC X(14) VALUE "Hybrid Tea".
000073       10  PIC X(14) VALUE "Floribunda".
000074       10  PIC X(14) VALUE "Old-fashioned".
000075       10  PIC X(14) VALUE "Modern Shrub".
000076       10  PIC X(14) VALUE "Climber".
000077       10  PIC X(14) VALUE "Rambler".
000078       10  PIC X(14) VALUE "Miniature".
000079       10  PIC X(14) VALUE "Species".
000080     05  REDEFINES TYPE-TABLE-DATA.
000081       10  TYPE-DESCR         PIC X(14) OCCURS 8 TIMES.
000082 01  COLOUR-TABLE.
000083     05  COLOUR-TABLE-DATA.
000084       10  VALUE "White YellowAmber ".
000085       10  VALUE "OrangePink  Red   ".
000086       10  VALUE "Violet".
000087     05  REDEFINES COLOUR-TABLE-DATA.
000088       10  COLOUR-DESCR       PIC X(6) OCCURS 7 TIMES.
000089
000090 01  SEASON-TABLE.
000091     05  SEASON-TABLE-DATA.
000092       10  VALUE "SpringSummerAutumnWinter".
000093     05  REDEFINES SEASON-TABLE-DATA.
000094       10  SEASON-DESCR       PIC X(6) OCCURS 4 TIMES.
000095
000096*    FORMAT DEFINITIONS
000097*    ------------------
000098*    These definitions require just over 5000 bytes in the
000099*    Data Division. This could become a problem in a much
000100*    larger program. The best solution is to define them
000101*    in a CALLed program, which allows 65K bytes for them.
000102
000103 01  MAIN-FORMAT.
000104     05  VALUE "|ANL|V Roses Database".
000105     05  VALUE "|C0603|B0206 F_ind".
000106     05  VALUE "|C0612|B0206 N_ext".
000107     05  VALUE "|C0621|B0207 P_rior".
000108     05  VALUE "|C0631|B0207 C_lear".
000109     05  VALUE "|C0641|B0205 A_dd".
000110     05  VALUE "|C0649|B0208 U_pdate".
000111     05  VALUE "|C0660|B0208 D_elete".
000112     05  VALUE "|C0671|B0206 eX_it".
000113     05  VALUE "|C0902|P0078".
000114     05  VALUE "|ABL".
000115     05  VALUE "|C1203Name|C1603Type|C2003Colour".
000116     05  VALUE "|C2403Height       meters|C2716feet".
000117     05  VALUE "|C3203Flowering season".
000118     05  VALUE "|C1211|F0222|C1611|F0216|C1625|D".
000119     05  VALUE "|C2011|F0208|C2017|D".
000120     05  VALUE "|C2411|F0204|C2711|F0204".
000121     05  VALUE "|C3511|F0208|C3517|D".
000122     05  VALUE "|C4411|F0847".
000123     05  VALUE "|X".
000124 01  TYPE-MENU-FORMAT.
000125     05  VALUE "|C1810|S2116".
000126     05  VALUE "|ANL|TF|C1911|W1914General type".
000127     05  VALUE "|C2211|F1614|X".
000128 01  COLOUR-MENU-FORMAT.
000129     05  VALUE "|C2210|S1910".
000130     05  VALUE "|ANL|TF|C2311|W1708Colour".
000131     05  VALUE "|C2611|F1408|X".
000132 01  SEASON-MENU-FORMAT.
000133     05  VALUE "|C3710|S1310".
000134     05  VALUE "|ANL|TF|C3811|W1108Season".
000135     05  VALUE "|C4111|F0808|X".
000136 01  SYSTEM-HELP-FORMAT.
000137     05  VALUE "|ANY|TC|Z|C1135|L2944".
000138     05  VALUE "|TF|C1245System description|TC|C1435".
000139     05  VALUE
000140         "|NThis is a database program to demonstrate".
000141     05  VALUE
000142         "|NCOBOL12, and its Format and Screen modules.".
000143     05  VALUE
000144         "|NThe database contains details of various ".
000145     05  VALUE
000146         "|Nroses, grouped by general type.".
000147     05  VALUE "|ANL|C2445".
000148     05  VALUE "|C2545|B0221 Help System".
000149     05  VALUE "|C2845|B0221 Enquiries".
000150     05  VALUE "|C3145|B0221 Adding Records".
000151     05  VALUE "|C3445|B0221 Updating Records".
000152     05  VALUE "|C3745|B0221 Deleting Records".
000153     05  VALUE "|TF|ANL|X".
000154 01  HELP-FORMAT.
000155     05  VALUE "|ANY|TC|Z|C1135|L2944".
000156     05  VALUE "|TF|C1250Help System|TC".
000157     05  VALUE "|ANL|C1272|B0206 Home|ANY|C1435".
000158     05  VALUE
000159         "|NThe Help system in this program provides a".
000160     05  VALUE
000161         "|Ngeneral description of the system and each".
000162     05  VALUE
000163         "|Nof its functions.|N".
000164     05  VALUE
000165         "|NIn addition, context-sensitive help is ".
000166     05  VALUE
000167         "|Navailable by right-clicking on any screen".
000168     05  VALUE
000169         "|Nfield or button.".
000170     05  VALUE "|TF|ANL|X".
000171 01  ENQUIRIES-HELP.
000172     05  VALUE "|ANY|TC|Z|C1135|L2944".
000173     05  VALUE "|TF|C1250Enquiries|TC".
000174     05  VALUE "|ANL|C1272|B0206 Home|ANY|C1435".
000175     05  VALUE
000176         "|NYou may enquire for a rose by name, by a".
000177     05  VALUE
000178         "|Ntype or by a colour. You can then scroll".
000179     05  VALUE
000180         "|Nback and forth through the records.".
000181     05  VALUE
000182         "|NIf type or colour is specified, only the".
000183     05  VALUE
000184         "|Nmatching roses will be scrolled.".
000185     05  VALUE
000186         "|NIf you enter a name, this is a starting".
000187     05  VALUE
000188         "|Npoint and all records will be scrolled.".
000189     05  VALUE
000190         "|NOnly the first few letters of a name need".
000191     05  VALUE
000192         "|Nbe entered to access the first matching".
000193     05  VALUE
000194         "|Nrecord, or the name may be left blank to".
000195     05  VALUE
000196         "|Naccess the first record in the file.".
000197     05  VALUE "|TF|ANL|X".
000198 01  ADDING-HELP.
000199     05  VALUE "|ANY|TC|Z|C1135|L2944".
000200     05  VALUE "|TF|C1245Adding New records|TC".
000201     05  VALUE "|ANL|C1272|B0206 Home|ANY|C1435".
000202     05  VALUE
000203         "|NTo add a new record, the details are entered".
000204     05  VALUE
000205         "|Non the screen, and then the A_dd button is".
000206     05  VALUE
000207         "|Nselected. The screen may first be cleared".
000208     05  VALUE
000209         "|Nwith the C_lear button, or the details of".
000210     05  VALUE
000211         "|Nthe new record may be entered over any".
000212     05  VALUE
000213         "|Nexisting details.".
000214     05  VALUE
000215         "|NEach record must have a unique name.".
000216     05  VALUE "|TF|ANL|X".
000217 01  UPDATING-HELP.
000218     05  VALUE "|ANY|TC|Z|C1135|L2944".
000219     05  VALUE "|TF|C1245Updating records|TC|C1335".
000220     05  VALUE "|ANL|C1272|B0206 Home|ANY|C1435".
000221     05  VALUE
000222         "|NTo update a record it must first be accessed".
000223     05  VALUE
000224         "|Nas for an enquiry. You may then amend any".
000225     05  VALUE
000226         "|Ndetails of that record, except for the name".
000227     05  VALUE
000228         "|Nwhich cannot be updated. The U_pdate button".
000229     05  VALUE
000230         "|Nis then selected.".
000231     05  VALUE
000232         "|NTo change the name, first delete the record,".
000233     05  VALUE
000234         "|Nthen change the name on the screen, and then".
000235     05  VALUE
000236         "|NA_dd the new, updated, record.".
000237     05  VALUE
000238         "|NIf you change a record and then leave it".
000239     05  VALUE
000240         "|Nwithout updating it in the file, the system".
000241     05  VALUE
000242         "|Nwill ask you if it is to be updated before".
000243     05  VALUE "|Nproceeding.".
000244     05  VALUE "|TF|ANL|X".
000245 01  DELETING-HELP.
000246     05  VALUE "|ANY|TC|Z|C1135|L2944".
000247     05  VALUE "|TF|C1245Deleting records|TC|C1335".
000248     05  VALUE "|ANL|C1272|B0206 Home|ANY|C1435".
000249     05  VALUE
000250         "|NTo delete a record it must first be accessed".
000251     05  VALUE
000252         "|Nas for an enquiry. The D_elete button is".
000253     05  VALUE
000254         "|Nthen selected. You will be asked to confirm".
000255     05  VALUE
000256         "|Nthe deletion before the record is removed.".
000257     05  VALUE
000258         "|NOnce deleted, the record still appears on".
000259     05  VALUE
000260         "|Nthe screen and you may A_dd it again as a".
000261     05  VALUE
000262         "|Nnew record.".
000263     05  VALUE "|TF|ANL|X".
000264 01  NAME-CONTEXT.
000265     05  VALUE "|ANC|TC|C1234|L0420".
000266     05  VALUE   "The specific, unique".
000267     05  VALUE "|Nname of this rose.".
000268     05  VALUE "|ANL|TF|X".
000269 01  DESCR-CONTEXT.
000270     05  VALUE "|ANC|TC|C4358|L0621".
000271     05  VALUE   "A general description".
000272     05  VALUE "|Nof the rose and its".
000273     05  VALUE "|Nmain characteristics.".
000274     05  VALUE "|ANL|TF|X".
000275 01  TYPE-CONTEXT.
000276     05  VALUE "|ANC|TC|C1628|L0624".
000277     05  VALUE   "The general type. Click".
000278     05  VALUE "|Nthe button to select, or".
000279     05  VALUE "|Nenter your own type.".
000280     05  VALUE "|ANL|TF|X".
000281 01  COLOUR-CONTEXT.
000282     05  VALUE "|ANC|TC|C2020|L0622".
000283     05  VALUE   "The rose colour. Click".
000284     05  VALUE "|Nthe button to select,".
000285     05  VALUE "|Nor use another colour.".
000286     05  VALUE "|ANL|TF|X".
000287 01  HEIGHT-CONTEXT.
000288     05  VALUE "|ANC|TC|C2423|L0822".
000289     05  VALUE   "The height in meters".
000290     05  VALUE "|Nand feet. Enter one".
000291     05  VALUE "|Nand the other will be".
000292     05  VALUE "|Ncalculated for you.".
000293     05  VALUE "|ANL|TF|X".
000294 01  SEASON-CONTEXT.
000295     05  VALUE "|ANC|TC|C3220|L0819".
000296     05  VALUE   "The main flowering".
000297     05  VALUE "|Nseason. Click the".
000298     05  VALUE "|Nbutton to select,".
000299     05  VALUE "|Nor enter your own.".
000300     05  VALUE "|ANL|TF|X".
000301 01  FIND-CONTEXT.
000302     05  VALUE "|ANC|TC|C0804|L0626".
000303     05  VALUE   "F_ind accesses a record".
000304     05  VALUE "|Nwhen you have entered a".
000305     05  VALUE "|Nname, a type, or a colour.".
000306     05  VALUE "|ANL|TF|X".
000307 01  NEXT-CONTEXT.
000308     05  VALUE "|ANC|TC|C0813|L0628".
000309     05  VALUE   "Use N_ext to get the next".
000310     05  VALUE "|Nrecord in sequence after".
000311     05  VALUE "|Nthe current record.".
000312     05  VALUE "|ANL|TF|X".
000313 01  PRIOR-CONTEXT.
000314     05  VALUE "|ANC|TC|C0822|L0623".
000315     05  VALUE   "Use P_rior to get the".
000316     05  VALUE "|Nrecord before the".
000317     05  VALUE "|Ncurrent record.".
000318     05  VALUE "|ANL|TF|X".
000319 01  CLEAR-CONTEXT.
000320     05  VALUE "|ANC|TC|C0832|L1027".
000321     05  VALUE   "C_lear MAY be used to".
000322     05  VALUE "|Nto clear the screen record".
000323     05  VALUE "|Nwhen entering a new record,".
000324     05  VALUE "|Nor name, type, or colour".
000325     05  VALUE "|Nfor a record to F_ind.".
000326     05  VALUE "|ANL|TF|X".
000327 01  ADD-CONTEXT.
000328     05  VALUE "|ANC|TC|C0842|L0825".
000329     05  VALUE   "When you have entered a".
000330     05  VALUE "|Nnew record on the screen,".
000331     05  VALUE "|Nuse A_dd to insert it in".
000332     05  VALUE "|Nthe file.".
000333     05  VALUE "|ANL|TF|X".
000334 01  UPDATE-CONTEXT.
000335     05  VALUE "|ANC|TC|C0850|L0825".
000336     05  VALUE   "U_pdate is used to replace".
000337     05  VALUE "|Nthe current record in the".
000338     05  VALUE "|Nfile when you have made".
000339     05  VALUE "|Nchanges to it.".
000340     05  VALUE "|ANL|TF|X".
000341 01  DELETE-CONTEXT.
000342     05  VALUE "|ANC|TC|C0855|L0825".
000343     05  VALUE   "When the current record".
000344     05  VALUE "|Nis no longer required,".
000345     05  VALUE "|Nuse D_elete to remove".
000346     05  VALUE "|Nit from the file.".
000347     05  VALUE "|ANL|TF|X".
000348 01  EXIT-CONTEXT.
000349     05  VALUE "|ANC|TC|C0856|L0624".
000350     05  VALUE   "X on the title line or".
000351     05  VALUE "|Nthe eX_it button will".
000352     05  VALUE "|Nterminate the program.".
000353     05  VALUE "|ANL|TF|X".
000354 01  NOT-FOUND-MESSAGE.
000355     05  VALUE "|M1 Find Record|N|N     Record not found".
000356     05  VALUE "|X".
000357 01  REWRITE-ERROR-MESSAGE.
000358     05  VALUE "|M1 Rewrite Record|N|N Status code ".
000359     05  REM-STATUS           PIC XX.
000360     05  VALUE "|N Invalid rewrite of Rose file".
000361     05  VALUE "|X".
000362 01  DELETE-WARNING-MESSAGE.
000363     05  VALUE "|M2 Deleting Record|N".
000364     05  VALUE "|N Do you wish to delete this".
000365     05  VALUE "|N record?|X".
000366 01  DELETED-MESSAGE.
000367     05  VALUE "|M1 Deleted Record|N".
000368     05  VALUE "|N This record has been deleted|X".
000369 01  NO-CURRENT-RECORD-MESSAGE.
000370     05  VALUE "|M1 Record Action|N".
000371     05  VALUE "|N There is no current record for".
000372     05  VALUE "|N this action to proceed.|X".
000373 01  NO-MORE-RECORDS-MESSAGE.
000374     05  VALUE "|M1 Record Action|N".
000375     05  VALUE "|N There are no more records for".
000376     05  VALUE "|N this action.|X".
000377 01  RECORD-CHANGED-MESSAGE.
000378     05  VALUE "|M2 Record Changed|N".
000379     05  VALUE "|N This record has been changed.".
000380     05  VALUE "|N Do you wish to save it now?|X".
000381 01  INVALID-NAME-CHANGE-MESSAGE.
000382     05  VALUE "|M1 Name Change|N".
000383     05  VALUE "|N You cannot change the name".
000384     05  VALUE "|N when updating. See the help".
000385     05  VALUE "|N behind this message.".
000386 01  END-OF-FORMATS.
000387     05  VALUE "|X".
000388
000389 SCREEN SECTION.
000390*--------------
000391 MAIN-SCREEN GROUP.
000392*-----------------
000393     CURR-NAME    AT 1211 UPPER.
000394     CURR-TYPE    AT 1611.
000395     CURR-COLOUR  AT 2011.
000396     CURR-METERS  AT 2411 BLANK ZERO RETURN.
000397     CURR-FEET    AT 2711 BLANK ZERO RETURN.
000398     CURR-SEASON  AT 3511.
000399     CURR-DESCR   AT 4411 FOR 0847.
000400     BUTTON AT 1625 FOR 0202.     *  8 Type
000401     BUTTON AT 2017 FOR 0202.     *  9 Colour
000402     BUTTON AT 3517 FOR 0202.     * 10 Season
000403     BUTTON AT 0603 FOR 0206 "F". * 11 Find
000404     BUTTON AT 0612 FOR 0206 "N". * 12 Next
000405     BUTTON AT 0621 FOR 0207 "P". * 13 Prior
000406     BUTTON AT 0631 FOR 0207 "C". * 14 Clear
000407     BUTTON AT 0641 FOR 0205 "A". * 15 Add
000408     BUTTON AT 0649 FOR 0207 "U". * 16 Update
000409     BUTTON AT 0660 FOR 0208 "D". * 17 Delete
000410     BUTTON AT 0671 FOR 0206 "X". * 18 eXit
000411     BUTTON AT 0278 FOR 0203.     * 19 Title X
000412     BUTTON AT 1272 FOR 0206.     * 20 Help home
000413     BUTTON AT 2545 FOR 0221.     * 21 Help Help
000414     BUTTON AT 2845 FOR 0221.     * 22 Help Enquiry
000415     BUTTON AT 3145 FOR 0221.     * 23 Help Adding
000416     BUTTON AT 3445 FOR 0221.     * 24 Help Updatin
000417     BUTTON AT 3745 FOR 0221.     * 25 Help Deletin
000418
000419 TYPE-DESCRS MENU.
000420*----------------
000421     TYPE-DESCR (1) AT 2211.
000422     TYPE-DESCR (2) AT 2411.
000423     TYPE-DESCR (3) AT 2611.
000424     TYPE-DESCR (4) AT 2811.
000425     TYPE-DESCR (5) AT 3011.
000426     TYPE-DESCR (6) AT 3211.
000427     TYPE-DESCR (7) AT 3411.
000428     TYPE-DESCR (8) AT 3611.
000429     BUTTON AT 1923 FOR 0202. * 9 = Exit
000430
000431 COLOUR-DESCRS MENU.
000432*------------------
000433     COLOUR-DESCR (1) AT 2612.
000434     COLOUR-DESCR (2) AT 2812.
000435     COLOUR-DESCR (3) AT 3012.
000436     COLOUR-DESCR (4) AT 3212.
000437     COLOUR-DESCR (5) AT 3412.
000438     COLOUR-DESCR (6) AT 3612.
000439     COLOUR-DESCR (7) AT 3812.
000440     BUTTON AT 2317 FOR 0202. * 8 = Exit
000441
000442 SEASON-DESCRS MENU.
000443*-----------------
000444     SEASON-DESCR  (1) AT 4111.
000445     SEASON-DESCR  (2) AT 4311.
000446     SEASON-DESCR  (3) AT 4511.
000447     SEASON-DESCR  (4) AT 4711.
000448     BUTTON AT 3817 FOR 0202. * 5 = Exit
000449
000450 MESSAGE-BOX GROUP.
000451*-----------------
000452     BUTTON AT 3430 FOR 0206. * 1 Yes
000453     BUTTON AT 3438 FOR 0206. * 2 OK
000454     BUTTON AT 3446 FOR 0206. * 3 No
000455     BUTTON AT 2156 FOR 0202. * 4 Exit - No
000456
000457 PROCEDURE DIVISION.
000458 MAIN-ROUTINE SECTION.
000459*--------------------
000460 MAIN-ENTRY.
000461     OPEN I-O ROSE-FILE
000462     DISPLAY FORMAT MAIN-FORMAT SYSTEM-HELP-FORMAT
000463     PERFORM CLEAR-RECORD
000464     PERFORM UNTIL EXIT
000465*      Permanent main loop
000466       ACCEPT SCREEN
000467       EVALUATE SCREEN-STATUS
000468       WHEN "X" EXIT PROGRAM
000469       WHEN "R" PERFORM HELP-ROUTINE
000470       WHEN "F" WHEN "E" PERFORM METERS-FEET-ROUTINE
000471       WHEN "L"
000472         EVALUATE SCREEN-POINTER
000473         WHEN  8 PERFORM TYPE-MENU
000474         WHEN  9 PERFORM COLOUR-MENU
000475         WHEN 10 PERFORM SEASON-MENU
000476         WHEN 11 PERFORM FIND-RECORD
000477         WHEN 12 PERFORM NEXT-RECORD
000478         WHEN 13 PERFORM PREVIOUS-RECORD
000479         WHEN 14 PERFORM CLEAR-RECORD
000480         WHEN 15 PERFORM ADD-RECORD
000481         WHEN 16 PERFORM UPDATE-RECORD
000482         WHEN 17 PERFORM DELETE-RECORD
000483         WHEN 18 WHEN 19 EXIT PROGRAM
000484         WHEN OTHER PERFORM HELP-ROUTINE
000485         END-EVALUATE
000486       END-EVALUATE
000487     END-PERFORM
000488*    An exit button has been clicked.
000489*    Check for updating before closing.
000490     PERFORM UPDATE-CHECK
000491     CLOSE ROSE-FILE
000492     STOP RUN
000493     .
000494 TYPE-MENU SECTION.
000495*-----------------
000496 TM-ENTRY.
000497     DISPLAY FORMAT TYPE-MENU-FORMAT
000498     SET SCREEN TO TYPE-DESCRS
000499     MOVE 1 TO SCREEN-POINTER
000500     PERFORM 8 TIMES
000501       IF CURR-TYPE = TYPE-DESCR (SCREEN-POINTER)
000502         EXIT PERFORM
000503       END-IF
000504       ADD 1 TO SCREEN-POINTER
000505     END-PERFORM
000506     IF SCREEN-POINTER > 8
000507       MOVE 1 TO SCREEN-POINTER
000508     END-IF
000509     ACCEPT SCREEN
000510     SET SCREEN TO MAIN-SCREEN
000511     IF SCREEN-FIELD NOT > 8
000512       MOVE TYPE-DESCR (SCREEN-FIELD) TO CURR-TYPE
000513     END-IF
000514     DISPLAY FORMAT "|R|X"
000515     MOVE 2 TO SCREEN-POINTER
000516*    EXIT
000517     .
000518 COLOUR-MENU SECTION.
000519*-------------------
000520 CM-ENTRY.
000521     DISPLAY FORMAT COLOUR-MENU-FORMAT
000522     SET SCREEN TO COLOUR-DESCRS
000523     MOVE 1 TO SCREEN-POINTER
000524     PERFORM 7 TIMES
000525       IF CURR-COLOUR = COLOUR-DESCR (SCREEN-POINTER)
000526         EXIT PERFORM
000527       END-IF
000528       ADD 1 TO SCREEN-POINTER
000529     END-PERFORM
000530     IF SCREEN-POINTER > 7
000531       MOVE 1 TO SCREEN-POINTER
000532     END-IF
000533     ACCEPT SCREEN
000534     SET SCREEN TO MAIN-SCREEN
000535     IF SCREEN-FIELD NOT > 7
000536       MOVE COLOUR-DESCR (SCREEN-FIELD) TO CURR-COLOUR
000537     END-IF
000538     DISPLAY FORMAT "|R|X"
000539     MOVE 3 TO SCREEN-POINTER
000540*    EXIT
000541     .
000542 SEASON-MENU SECTION.
000543*------------------
000544 SM-ENTRY.
000545     DISPLAY FORMAT SEASON-MENU-FORMAT
000546     SET SCREEN TO SEASON-DESCRS
000547     MOVE 1 TO SCREEN-POINTER
000548     PERFORM 4 TIMES
000549       IF CURR-SEASON = SEASON-DESCR (SCREEN-POINTER)
000550         EXIT PERFORM
000551       END-IF
000552       ADD 1 TO SCREEN-POINTER
000553     END-PERFORM
000554     IF SCREEN-POINTER > 4
000555       MOVE 1 TO SCREEN-POINTER
000556     END-IF
000557     ACCEPT SCREEN
000558     SET SCREEN TO MAIN-SCREEN
000559     IF SCREEN-FIELD NOT > 4
000560       MOVE SEASON-DESCR (SCREEN-FIELD) TO CURR-SEASON
000561     END-IF
000562     DISPLAY FORMAT "|R|X"
000563     MOVE 6 TO SCREEN-POINTER
000564*    EXIT
000565     .
000566 FIND-RECORD SECTION.
000567*-------------------
000568*    Find the first record matching the type, or the
000569*    colour if no type, or else the first record not
000570*    less than the name field.
000571 FR-ENTRY.
000572     SET NO-CURRENT-RECORD TO TRUE
000573     MOVE CURRENT-ROSE-RECORD TO ROSE-RECORD
000574     IF ROSE-TYPE NOT = SPACES
000575       START ROSE-FILE KEY NOT < ROSE-TYPE
000576         INVALID KEY CONTINUE
000577         ELSE SET CURRENT-RECORD TO TRUE
000578              MOVE "T" TO FIND-KEY
000579       END-START
000580     ELSE
000581       IF ROSE-COLOUR NOT = SPACES
000582         START ROSE-FILE KEY NOT < ROSE-COLOUR
000583           INVALID KEY CONTINUE
000584           ELSE SET CURRENT-RECORD TO TRUE
000585                MOVE "C" TO FIND-KEY
000586         END-START
000587       ELSE
000588         START ROSE-FILE KEY NOT < ROSE-NAME
000589           INVALID KEY
000590             CONTINUE
000591           ELSE SET CURRENT-RECORD TO TRUE
000592              MOVE "N" TO FIND-KEY
000593         END-START
000594       END-IF
000595     END-IF
000596     IF CURRENT-RECORD
000597          PERFORM NEXT-RECORD
000598     ELSE DISPLAY FORMAT NOT-FOUND-MESSAGE
000599          PERFORM GET-MESSAGE-REPLY
000600     END-IF
000601*    EXIT
000602     .
000603 CLEAR-RECORD SECTION.
000604*--------------------
000605*    Clear the record areas and redisplay the screen.
000606 CR-ENTRY.
000607     MOVE SPACES TO CURRENT-ROSE-RECORD
000608     MOVE ZERO TO CURR-METERS CURR-FEET
000609     MOVE CURR-HEIGHT TO SAVED-HEIGHT
000610     MOVE CURRENT-ROSE-RECORD TO ROSE-RECORD
000611     SET NO-CURRENT-RECORD TO TRUE
000612     MOVE 1 TO SCREEN-POINTER
000613     DISPLAY SCREEN
000614*    EXIT
000615     .
000616 ADD-RECORD SECTION.
000617*------------------
000618 ADD-ENTRY.
000619     MOVE CURRENT-ROSE-RECORD TO ROSE-RECORD
000620     WRITE ROSE-RECORD INVALID KEY
000621          SET SCREEN COMMENT TO "Duplicate name"
000622          DISPLAY FORMAT ADDING-HELP
000623     ELSE SET CURRENT-RECORD TO TRUE
000624          MOVE "N" TO FIND-KEY
000625     END-WRITE
000626*    EXIT
000627     .
000628 UPDATE-RECORD SECTION.
000629*---------------------
000630 UPD-ENTRY.
000631     IF CURRENT-RECORD
000632       IF ROSE-NAME = CURR-NAME
000633         MOVE CURRENT-ROSE-RECORD TO ROSE-RECORD
000634         REWRITE ROSE-RECORD INVALID KEY
000635           MOVE ROSE-STATUS TO REM-STATUS
000636           DISPLAY FORMAT REWRITE-ERROR-MESSAGE
000637           PERFORM GET-MESSAGE-REPLY
000638         END-REWRITE
000639       ELSE
000640         DISPLAY FORMAT UPDATING-HELP
000641                        INVALID-NAME-CHANGE-MESSAGE
000642         PERFORM GET-MESSAGE-REPLY
000643         MOVE 1 TO SCREEN-POINTER
000644       END-IF
000645     ELSE
000646       DISPLAY FORMAT NO-CURRENT-RECORD-MESSAGE
000647       PERFORM GET-MESSAGE-REPLY
000648     END-IF
000649*    EXIT
000650     .
000651 DELETE-RECORD SECTION.
000652*---------------------
000653 DEL-ENTRY.
000654     PERFORM UNTIL EXIT
000655       IF NO-CURRENT-RECORD
000656         DISPLAY FORMAT NO-CURRENT-RECORD-MESSAGE
000657         PERFORM GET-MESSAGE-REPLY
000658         EXIT PERFORM
000659       END-IF
000660       DISPLAY FORMAT DELETE-WARNING-MESSAGE
000661       PERFORM GET-MESSAGE-REPLY
000662       IF MESSAGE-REPLY NOT = 1  * Yes
000663         EXIT PERFORM
000664       END-IF
000665       DELETE ROSE-FILE RECORD INVALID KEY
000666         EXIT PERFORM
000667       END-DELETE
000668       SET DELETED-RECORD TO TRUE
000669       DISPLAY FORMAT DELETED-MESSAGE
000670       PERFORM GET-MESSAGE-REPLY
000671       EXIT PERFORM
000672     END-PERFORM
000673*    EXIT
000674     .
000675 PREVIOUS-RECORD SECTION.
000676*-----------------------
000677 PRV-ENTRY.
000678     IF NO-CURRENT-RECORD
000679       DISPLAY FORMAT NO-CURRENT-RECORD-MESSAGE
000680       PERFORM GET-MESSAGE-REPLY
000681     ELSE
000682       PERFORM UPDATE-CHECK
000683       READ ROSE-FILE PRIOR RECORD AT END
000684         DISPLAY FORMAT NO-MORE-RECORDS-MESSAGE
000685         PERFORM GET-MESSAGE-REPLY
000686       ELSE
000687         MOVE ROSE-RECORD TO CURRENT-ROSE-RECORD
000688         MOVE CURR-HEIGHT TO SAVED-HEIGHT
000689         MOVE 1 TO SCREEN-POINTER
000690         DISPLAY SCREEN
000691       END-READ
000692     END-IF
000693*    EXIT
000694     .
000695 NEXT-RECORD SECTION.
000696*-------------------
000697*    Read the next record in index sequence until the
000698*    end of file is reached.
000699 NXT-ENTRY.
000700     IF NO-CURRENT-RECORD
000701       DISPLAY FORMAT NO-CURRENT-RECORD-MESSAGE
000702       PERFORM GET-MESSAGE-REPLY
000703     ELSE
000704       PERFORM UNTIL EXIT
000705         PERFORM UPDATE-CHECK
000706         READ ROSE-FILE NEXT RECORD AT END CONTINUE
000707         ELSE
000708           IF FIND-KEY = "T" AND ROSE-TYPE = CURR-TYPE
000709           OR FIND-KEY = "C" AND ROSE-COLOUR = CURR-COLOUR
000710           OR FIND-KEY = "N"
000711             MOVE ROSE-RECORD TO CURRENT-ROSE-RECORD
000712             MOVE CURR-HEIGHT TO SAVED-HEIGHT
000713             MOVE 1 TO SCREEN-POINTER
000714             DISPLAY SCREEN
000715             EXIT PERFORM
000716           END-IF
000717         END-READ
000718         DISPLAY FORMAT NO-MORE-RECORDS-MESSAGE
000719         PERFORM GET-MESSAGE-REPLY
000720         MOVE CURRENT-ROSE-RECORD TO ROSE-RECORD
000721         EXIT PERFORM
000722       END-PERFORM
000723     END-IF
000724*    EXIT
000725     .
000726 METERS-FEET-ROUTINE SECTION.
000727*---------------------------
000728*    Calculate meters to feet or feet to meters.
000729 MFR-ENTRY.
000730     IF CURR-HEIGHT NOT = SAVED-HEIGHT
000731       IF SCREEN-FIELD = 4  * Meters
000732            MULTIPLY CURR-METERS BY 3.281
000733                     GIVING CURR-FEET ROUNDED
000734            MOVE 5 TO SCREEN-POINTER
000735            DISPLAY SCREEN FIELD
000736       ELSE IF CURR-FEET NOT ZERO
000737                 DIVIDE CURR-FEET BY 3.281
000738                        GIVING CURR-METERS ROUNDED
000739            ELSE MOVE ZERO TO CURR-METERS
000740            END-IF
000741            MOVE 4 TO SCREEN-POINTER
000742            DISPLAY SCREEN FIELD
000743       END-IF
000744       MOVE CURR-HEIGHT TO SAVED-HEIGHT
000745     END-IF
000746*    EXIT
000747     .
000748 HELP-ROUTINE SECTION.
000749*--------------------
000750*    Process all help requests, then wait for a screen
000751*    response before restoring the screen and returning
000752*    to the main routine.
000753 HR-ENTRY.
000754     EVALUATE SCREEN-POINTER
000755     WHEN  1 DISPLAY FORMAT NAME-CONTEXT
000756     WHEN  2 WHEN  8 DISPLAY FORMAT TYPE-CONTEXT
000757     WHEN  3 WHEN  9 DISPLAY FORMAT COLOUR-CONTEXT
000758     WHEN  4 WHEN  5 DISPLAY FORMAT HEIGHT-CONTEXT
000759     WHEN  6 WHEN 10 DISPLAY FORMAT SEASON-CONTEXT
000760     WHEN  7 DISPLAY FORMAT DESCR-CONTEXT
000761     WHEN 11 DISPLAY FORMAT FIND-CONTEXT
000762     WHEN 12 DISPLAY FORMAT NEXT-CONTEXT
000763     WHEN 13 DISPLAY FORMAT PRIOR-CONTEXT
000764     WHEN 14 DISPLAY FORMAT CLEAR-CONTEXT
000765     WHEN 15 DISPLAY FORMAT ADD-CONTEXT
000766     WHEN 16 DISPLAY FORMAT UPDATE-CONTEXT
000767     WHEN 17 DISPLAY FORMAT DELETE-CONTEXT
000768     WHEN 18 WHEN 19 DISPLAY FORMAT EXIT-CONTEXT
000769     WHEN 20 DISPLAY FORMAT SYSTEM-HELP-FORMAT
000770     WHEN 21 DISPLAY FORMAT HELP-FORMAT
000771     WHEN 22 DISPLAY FORMAT ENQUIRIES-HELP
000772     WHEN 23 DISPLAY FORMAT ADDING-HELP
000773     WHEN 24 DISPLAY FORMAT UPDATING-HELP
000774     WHEN 25 DISPLAY FORMAT DELETING-HELP
000775     END-EVALUATE
000776     IF SCREEN-POINTER NOT > 19
000777       MOVE ZERO TO SCREEN-POINTER
000778*      Zero accepts no data, only a control or mouse click
000779       ACCEPT SCREEN FIELD
000780       DISPLAY FORMAT "|R|X"
000781     ELSE MOVE SCREEN-FIELD TO SCREEN-POINTER
000782     END-IF
000783*    EXIT
000784     .
000785 UPDATE-CHECK SECTION.
000786*--------------------
000787*    This is a check to see if the current record has
000788*    been updated before it is lost by the next action.
000789 UCHK-ENTRY.
000790     IF CURRENT-RECORD
000791       AND ROSE-RECORD NOT = CURRENT-ROSE-RECORD
000792       DISPLAY FORMAT RECORD-CHANGED-MESSAGE
000793       PERFORM GET-MESSAGE-REPLY
000794       IF MESSAGE-REPLY = 1  * Yes
000795         PERFORM UPDATE-RECORD
000796       END-IF
000797     END-IF
000798*    EXIT
000799     .
000800 GET-MESSAGE-REPLY SECTION.
000801*-------------------------
000802*    Get the reply from a message box, and set the
000803*    MESSAGE-REPLY to 1, 2, 3, 4 for Yes, OK, No, Exit.
000804*    In a program with multiple screens, you would
000805*    need to have the user routine reset the screen group,
000806*    but that is done here, with only one screen.
000807 GMR-ENTRY.
000808     SET SCREEN TO MESSAGE-BOX
000809     MOVE ZERO TO SCREEN-POINTER
000810     ACCEPT SCREEN
000811     MOVE SCREEN-POINTER TO MESSAGE-REPLY
000812     SET SCREEN TO MAIN-SCREEN
000813     DISPLAY FORMAT "|R|X" * Remove message
000814*    EXIT
000815     .
